"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const config_1 = require("./config");
const symbols_1 = require("./symbols");
const utils_1 = require("./utils");
const valit_1 = require("./valit");
const vality_1 = require("./vality");
vality_1.vality.array = (0, valit_1.valit)("array", e => (value, path, options) => {
    const fn = (0, utils_1.enyToGuardFn)(e);
    if (!Array.isArray(value)) {
        if (!config_1.config.strict) {
            const res = fn(value, [...path, 0], value);
            if (res.valid) {
                return { valid: true, data: [res.data], errors: [] };
            }
        }
        return { valid: false, data: undefined, errors: [{ message: "vality.array.base", path, options, value }] };
    }
    const data = [];
    const errors = [];
    for (const k in value) {
        const res = fn(value[k], [...path, k], value);
        if (!res.valid) {
            errors.push(...res.errors);
            if (options.bail)
                break;
        }
        else {
            data.push(res.data);
        }
    }
    if (errors.length === 0)
        return { valid: true, data, errors: [] };
    return { valid: false, data: undefined, errors };
}, {
    minLength: (val, o) => val.length >= o,
    maxLength: (val, o) => val.length <= o,
}, {
    bail: false,
});
vality_1.vality.object = (0, valit_1.valit)("object", e => (value, path, options) => {
    if (typeof value !== "object" || value === null)
        return { valid: false, data: undefined, errors: [{ message: "vality.object.base", path, options, value }] };
    const data = {};
    const errors = [];
    for (const k in e) {
        const ek = e[k];
        if (typeof ek === "object" && ek !== null && symbols_1._readonly in ek)
            continue;
        const res = (0, utils_1.enyToGuardFn)(ek)(value[k], [...path, k], value);
        if (!res.valid) {
            errors.push(...res.errors);
            if (options.bail)
                break;
        }
        else {
            data[k] = res.data;
        }
    }
    for (const k in value) {
        const ek = e[k];
        if (ek === undefined || (typeof ek === "object" && ek !== null && symbols_1._readonly in ek)) {
            errors.push({
                message: "vality.object.extraProperty",
                path: [...path, k],
                options,
                value,
            });
            if (options.bail)
                break;
        }
    }
    if (errors.length === 0)
        return { valid: true, data: data, errors: [] };
    return { valid: false, data: undefined, errors };
}, {}, {
    bail: false,
});
vality_1.vality.optional = (0, valit_1.valit)("optional", e => (val, path) => {
    const enyVal = (0, utils_1.enyToGuardFn)(e)(val, path);
    if (enyVal.valid)
        return enyVal;
    if (val === undefined)
        return { valid: true, data: undefined, errors: [] };
    return enyVal;
});
vality_1.vality.enum = (0, valit_1.valit)("enum", (...es) => (value, path, options) => {
    for (const e of es) {
        const res = (0, utils_1.enyToGuardFn)(e)(value, path);
        if (res.valid)
            return res;
    }
    return { valid: false, data: undefined, errors: [{ message: "vality.enum.base", path, options, value }] };
});
vality_1.vality.tuple = (0, valit_1.valit)("tuple", (...es) => (value, path, options) => {
    if (!Array.isArray(value) || value.length !== es.length)
        return { valid: false, data: undefined, errors: [{ message: "vality.tuple.base", path, options, value }] };
    const data = [];
    const errors = [];
    for (let i = 0; i < es.length; i++) {
        const res = (0, utils_1.enyToGuardFn)(es[i])(value[i], [...path, i], es);
        if (!res.valid) {
            errors.push(...res.errors);
            if (options.bail)
                break;
        }
        else {
            data[i] = res.data;
        }
    }
    for (let i = es.length; i < value.length; i++) {
        errors.push({
            message: "vality.tuple.extraProperty",
            path: [...path, i],
            options,
            value,
        });
        if (options.bail)
            break;
    }
    if (errors.length === 0)
        return { valid: true, data: data, errors: [] };
    return { valid: false, data: undefined, errors };
}, {}, {
    bail: false,
});
vality_1.vality.readonly = () => ({
    [symbols_1._readonly]: true,
    [symbols_1._validate]: (value, path) => value === undefined
        ? { valid: true, data: undefined }
        : {
            valid: false,
            errors: [
                {
                    message: "vality.readonly.base",
                    path,
                    options: {},
                    value,
                },
            ],
        },
});
